@@ -10,7 +10,6 @@
 import uuid
 import re
 import tempfile
-import json
 import warnings
 import time
 import dateutil.parser
@@ -22,6 +21,7 @@ from itertools import chain, cycle
 from mimetypes import guess_type
 from email.utils import parseaddr
 
+import simplejson as json
 from babel.dates import parse_date
 from nereid import (
     request, abort, render_template, login_required, url_for, redirect,
@@ -54,9 +54,18 @@ PROGRESS_STATES = [
     ('Planning', 'Planning'),
     ('In Progress', 'In Progress'),
     ('Review', 'Review/QA'),
+    ('Done', 'Done'),
 ]
 
 
+def request_wants_json():
+    best = request.accept_mimetypes \
+        .best_match(['application/json', 'text/html'])
+    return best == 'application/json' and \
+        request.accept_mimetypes[best] > \
+        request.accept_mimetypes['text/html']
+
+
 class WebSite:
     """
     Website
@@ -90,7 +99,7 @@ class ProjectUsers(ModelSQL):
         super(ProjectUsers, cls).__setup__()
         cls._sql_constraints += [(
             'check_user',
-            'UNIQUE(project, user)',
+            'UNIQUE(project, "user")',
             'Users must be unique per project'
         )]
 
@@ -269,6 +278,24 @@ class Project:
     """
     __name__ = 'project.work'
 
+    # This function field returns the project of a task even if the tree
+    # involves a much deeper subtask structure
+    project = fields.Function(fields.Many2One(
+        'project.work', 'Project',
+        domain=[
+            ('type', '=', 'project')
+        ], states={'invisible': Eval('type') != 'project'},
+    ), 'get_parent_project')
+
+    def get_parent_project(self, name):
+        """
+        Gets the parent of this project
+        """
+        # TODO: Make this function work even with highly nested tasks
+        if self.parent:
+            return self.parent.id
+        return self.id
+
     history = fields.One2Many(
         'project.work.history', 'project',
         'History', readonly=True
@@ -397,6 +424,8 @@ class Project:
             value['hours'] = self.hours
             value['effort'] = self.effort
             value['total_effort'] = self.total_effort
+            value['project'] = self.project and self.project.id
+            value['created_by'] = self.created_by and self.created_by._json()
         else:
             value['all_participants'] = [
                 participant._json() for participant in self.all_participants
@@ -414,6 +443,7 @@ class Project:
             'type': self.type,
             'objectType': self.__name__,
         }
+        rv.update(self.serialize())
         if self.type == 'project':
             rv['url'] = url_for(
                 'project.work.render_project', project_id=self.id
@@ -1058,7 +1088,7 @@ class Project:
             'project/global-task-list.jinja',
             active_type_name='render_task_list', counts=counts,
             state_filter=state, tasks_by_state=tasks_by_state,
-            states=PROGRESS_STATES
+            states=PROGRESS_STATES[:-1]
         )
 
     @classmethod
@@ -1081,7 +1111,10 @@ class Project:
 
         if request.is_xhr:
             response = cls.serialize(task)
-            response['comments'] = [comment._json() for comment in comments]
+            with Transaction().set_context(task=task_id):
+                response['comments'] = [
+                    comment._json() for comment in comments
+                ]
             return jsonify(response)
 
         return render_template(
@@ -1510,7 +1543,7 @@ class Project:
             'project/tasks-by-employee.jinja',
             tasks_by_employee_by_state=tasks_by_employee_by_state,
             employees=employees,
-            states=PROGRESS_STATES,
+            states=PROGRESS_STATES[:-1],
         )
 
     @classmethod
@@ -1658,15 +1691,16 @@ class Project:
         attached_file = request.files["file"]
         resource = '%s,%d' % (cls.__name__, work.id)
 
+        filename = attached_file.filename
         if Attachment.search([
-            ('name', '=', attached_file.filename),
+            ('name', '=', filename),
             ('resource', '=', resource)
         ]):
-            flash(
-                'File already exists with same name, please choose another ' +
-                'file or rename this file to upload !!'
+            # try to create a unique filename
+            filename, extension = filename.split('.', 1)
+            filename = '%s-%d.%s' % (
+                filename, time.time(), extension
             )
-            return redirect(request.referrer)
 
         data = {
             'resource': resource,
@@ -1683,16 +1717,18 @@ class Project:
         else:
             data.update({
                 'data': attached_file.stream.read(),
-                'name': attached_file.filename,
+                'name': filename,
                 'type': 'data'
             })
 
-        Attachment.create(data)
+        attachment = Attachment.create(data)
 
-        if request.is_xhr:
-            return jsonify({
-                'success': True
-            })
+        if request.is_xhr or request_wants_json():
+            with Transaction().set_context(task=work.id):
+                return jsonify({
+                    'success': True,
+                    'data': attachment._json(),
+                })
 
         flash("Attachment added to %s" % work.name)
         return redirect(request.referrer)
@@ -1717,7 +1753,7 @@ class Project:
             'comment': request.form['comment']
         }
 
-        updatable_attrs = ['state', 'progress_state']
+        updatable_attrs = ['progress_state']
         new_participant_ids = []
         current_participant_ids = [p.id for p in task.participants]
         post_attrs = [request.form.get(attr, None) for attr in updatable_attrs]
@@ -1729,6 +1765,11 @@ class Project:
                 if getattr(task, attr) != request.form.get(attr, None):
                     task_changes[attr] = request.form[attr]
 
+            if task_changes.get('progress_state') == 'Done':
+                task_changes['state'] = 'done'
+            else:
+                task_changes['state'] = 'opened'
+
             new_assignee_id = request.form.get('assigned_to', None, int)
             if not new_assignee_id is None:
                 if (new_assignee_id and
@@ -2066,7 +2107,9 @@ class Project:
         page = request.args.get('page', 1, int)
 
         domain = [('project', '=', self.id)]
-        activities = Pagination(Activity, domain, page, 20)
+        activities = Pagination(
+            Activity, domain, page, request.args.get('limit', 20, type=int)
+        )
         items = filter(
             None, map(lambda activity: activity.serialize(), activities)
         )
@@ -2283,6 +2326,7 @@ class ProjectHistory(ModelSQL, ModelView):
             "comment": self.comment,
             "new_state": self.new_state,
             "new_progress_state": self.new_progress_state,
+            "previous_progress_state": self.previous_progress_state,
             "new_assignee": (
                 self.new_assigned_to._json() if self.new_assigned_to
                     else None
@@ -2483,7 +2527,7 @@ class ProjectWorkCommit(ModelSQL, ModelView):
             "displayName": self.commit_message,
             "repository": self.repository,
             "repository_url": self.repository_url,
-            "commit_timestamp": self.commit_timestamp,
+            "commit_timestamp": self.commit_timestamp.isoformat(),
             "commit_id": self.commit_id,
         }
 
@@ -2669,9 +2713,18 @@ class Attachment:
     __name__ = "ir.attachment"
 
     def _json(self):
-        return {
+        rv = {
             'create_date': self.create_date.isoformat(),
             "objectType": self.__name__,
             "id": self.id,
             "updatedBy": self.uploaded_by._json(),
+            "displayName": self.name,
+            "description": self.description,
         }
+        if has_request_context():
+            rv['downloadUrl'] = url_for(
+                'project.work.download_file',
+                attachment_id=self.id,
+                task=Transaction().context.get('task'),
+            )
+        return rv
