@@ -10,11 +10,13 @@
 import unittest
 import json
 import smtplib
+import pytz
 
 from trytond.config import CONFIG
 CONFIG['smtp_from'] = 'test@openlabs.co.in'
 CONFIG['data_path'] = '.'
 from minimock import Mock
+from datetime import datetime
 
 import trytond.tests.test_tryton
 from trytond.tests.test_tryton import POOL, DB_NAME, USER, CONTEXT
@@ -57,6 +59,8 @@ class TestTask(NereidTestCase):
         self.Tag = POOL.get('project.work.tag')
         self.History = POOL.get('project.work.history')
         self.Permission = POOL.get('nereid.permission')
+        self.ProjectWorkCommit = POOL.get('project.work.commit')
+        self.Activity = POOL.get('nereid.activity')
         self.xhr_header = [
             ('X-Requested-With', 'XMLHttpRequest'),
         ]
@@ -224,7 +228,8 @@ class TestTask(NereidTestCase):
             'company': data['company'].id,
         })
 
-        self.Project.write([data['task1'].parent],
+        self.Project.write(
+            [data['task1'].parent],
             {
                 'participants': [
                     ('add', [
@@ -254,10 +259,10 @@ class TestTask(NereidTestCase):
             'localhost/project/emails/html_content.jinja': '',
             'localhost/project/task.jinja': '{{ task.id }}',
             'localhost/project/comment.jinja': '',
-            'localhost/project/tasks-by-employee.jinja':
-                '',
-            'localhost/project/project-task-list.jinja':
-                '{{ tasks|length }}',
+            'localhost/project/tasks-by-employee.jinja': '',
+            'localhost/project/project-task-list.jinja': '{{ tasks|length }}',
+            'localhost/project/emails/project_text_content.jinja': '{{ task }}',
+            'localhost/project/emails/project_html_content.jinja': '{{ task }}',
         }
         return self.templates.get(name)
 
@@ -776,7 +781,9 @@ class TestTask(NereidTestCase):
 
                     # Login Success
                     self.assertEqual(response.status_code, 302)
-                    self.assertEqual(response.location, 'http://localhost/en_US/')
+                    self.assertEqual(
+                        response.location, 'http://localhost/en_US/'
+                    )
 
                     # Mark time when user is not employee
                     response = c.post(
@@ -1115,6 +1122,74 @@ class TestTask(NereidTestCase):
                         response.data
                     )
 
+    def test_0210_github_commit_activity_stream(self):
+        """
+        Checks activity stream generation for commit message and github hook
+        handler
+        """
+        with Transaction().start(DB_NAME, USER, CONTEXT):
+            data = self.create_task_dafaults()
+            app = self.get_app()
+            task = data['task1']
+
+            login_data = {
+                'email': 'email@example.com',
+                'password': 'password',
+            }
+            utc = pytz.UTC
+
+            payload = {
+                'commits': [{
+                    'author': {'email': 'email@example.com'},
+                    'message': 'Add commit #%d' % task.id,
+                    'timestamp': str(utc.localize(datetime.utcnow())),
+                    'url': 'repo/url/1',
+                    'id': '54321',
+                }],
+                'repository': {
+                    'name': 'ABC Repository',
+                    'url': 'repo/url',
+                }
+            }
+
+            with app.test_client() as c:
+                response = c.post('/en_US/login', data=login_data)
+
+                # Login Success
+                self.assertEqual(response.status_code, 302)
+
+                with Transaction().set_context(
+                    {'company': data['company'].id}
+                ):
+                    self.assertEqual(
+                        len(data['registered_user1'].activities), 0
+                    )
+
+                    # Check github handler
+                    response = c.post(
+                        '/en_US/-project/-github-hook',
+                        data={
+                            'payload': json.dumps(payload)
+                        }
+                    )
+                    self.assertEqual(response.status_code, 200)
+                    self.assertTrue(response.data, 'OK')
+
+                    # Activity stream is created for commit user
+                    self.assertEqual(
+                        len(data['registered_user1'].activities), 1
+                    )
+
+                    commit, = self.ProjectWorkCommit.search([
+                        ('commit_id', '=', '54321')
+                    ])
+
+                    activities = self.Activity.search([
+                        ('object_', '=', 'project.work.commit, %d' % commit.id)
+                    ]),
+
+                    self.assertEqual(len(activities), 1)
+
 
 def suite():
     "Nereid test suite"
